<?php


include_once "oai2/DcMetadataRecord.php";
include_once "oai2/MetadataRecordReader.php";
include_once "oai2/DublinCore.php";



class Mek2DcRecordIterator extends RecordIterator {


    function Mek2DcRecordIterator($dataSource, $theQuery, $theResult, $skipRows) {
	$this->dataSource   = $dataSource;
	$this->theQuery     = $theQuery;
	$this->skipRows     = $skipRows;

	if ($this->skipRows) {
	    $this->skipOnNext = true;
	} else {
	    $this->skipOnNext = false;
	}
	
	//var_dump($theQuery);

	// Store a database connection to query single records and
	// sets, so we dont have to connect every time a
	// getNextElement() call is made!
	$this->entityDataBase = DB::connect($dataSource);
	$this->entityDataBase->setFetchMode(DB_FETCHMODE_ASSOC);


	// There is no result. So we redo the query on the given
	// database, to get back the result.  This may lead to a
	// different result if the database was changed. Therefore no
	// resumption tokens should be issued in this time, or all
	// outstanding resumption tokens should be invalidated.
	if ($theResult == NULL) {

	    $dataBase = DB::connect($dataSource);
	    $dataBase->setFetchMode(DB_FETCHMODE_ASSOC);

	    $theResult = $dataBase->query($this->theQuery);
	    if (DB::isError($theResult)) {
		die($theResult->getMessage());
	    }

	    $numberOfRows = $theResult->numRows();
	    if (DB::isError($numberOfRows)) {
		die($numberOfRows->getMessage());
	    }
	}

	$this->theResult    = $theResult;
	$this->numberOfRows = $theResult->numRows();
	if (DB::isError($numberOfRows)) {
	    die($numberOfRows->getMessage());
	}
	// We will skip $skipRows rows at the first invocation of
	// getNextElement.  Therefore the number of rows available to
	// the user is reduced.
	$this->numberOfRows -= $this->skipRows;

	// The total number of rows given should not change between
	// successive calls, as long as we dont update the
	// database. Therefore the database should not be updated
	// while valid resumption tokens exist.
	$this->totalNumberOfRows = $theResult->numRows();

	if ($this->totalRows == 0) {
	    $this->totalRows = $this->numberOfRows;
	}
    }


    // Return the number of elements (this is identical to elements,
    // if no skipRows are given).
    function elements() {
	return $this->numberOfRows;
    }


    function totalElements() {
	return $this->totalNumberOfRows;
    }


    function readElements() {
	return $this->skipRows;
    }


    function hasMoreElements() {
	return $this->numberOfRows > 0;
    }



    function doQuery($query) {

	$recordResult = $this->entityDataBase->query($query);
	if (DB::isError($recordResult)) {
	    die($recordResult->getMessage());
	}

	$numberOfRows = $recordResult->numRows();
	if (DB::isError($numberOfRows)) {
	    die($numberOfRows->getMessage());
	}

	$theResult = array();
	for ($count = 0; $count < $numberOfRows; $count++) {
	    $aRow = $recordResult->fetchRow();
	    if (DB::isError($aRow)) {
		die($aRow->getMessage());
	    }
	    array_push($theResult, $aRow);
	}
	return $theResult;
    }



    function getRows($table, $field, $value, $where = "") {

	if (is_string($value)) {
	    $query = "select * from $table where $field = \"$value\"";
	} else {
	    $query = "select * from $table where $field = $value";
	}

	if ($where) {
	    $query .= " and $where";
	}
	return $this->doQuery($query);
    }



    function getRowsForId($aTable, $anId, $where = "") {

	return $this->getRows($aTable, "id", $anId, $where);
    }



    function insertTitle(&$aRecord, $anId) {	

	$theRows = $this->getRowsForId("dc_title", $anId);
	foreach ($theRows as $value) {
	    if ($value["subtitle"]) {
		$aRecord->addEntryAsUtf8("title", $value["main"] . " --- " . $value["subtitle"]);
	    } else {
		$aRecord->addEntryAsUtf8("title", $value["main"]);
	    }
	}
    }



    function insertCreator(&$aRecord, $anId) {	

	$theRows = $this->getRowsForId("dc_creator", $anId);
	foreach ($theRows as $value) {
	    if ($value["OtherName"]) {
		$aRecord->addEntryAsUtf8("creator", $value["OtherName"] . " " . $value["GivenName"] . " " . $value["FamilyName"]);
	    } else {
		$aRecord->addEntryAsUtf8("creator", $value["GivenName"] . " " . $value["FamilyName"]);
	    }
	}
    }



    function insertContributor(&$aRecord, $anId) {	

	$theRows = $this->getRowsForId("dc_contributor", $anId);
	foreach ($theRows as $value) {
	    if ($value["OtherName"]) {
		$aRecord->addEntryAsUtf8("contributor", $value["OtherName"] . " " . $value["GivenName"] . " " . $value["FamilyName"]);
	    } else {
		$aRecord->addEntryAsUtf8("contributor", $value["GivenName"] . " " . $value["FamilyName"]);
	    }
	}
    }



    function insertDate(&$aRecord, $anId) {	

	$theRows = $this->getRowsForId("dc_date", $anId);
	foreach ($theRows as $value) {
	    if ($value["mekevent"]) {
		$aRecord->addEntryAsUtf8("date", $value["date"] . " (" . $value["mekevent"] . ")");
	    } else {
		$aRecord->addEntryAsUtf8("date", $value["date"]);
	    }
	}
    }



    function insertPublisher(&$aRecord, $anId) {	

	$theRows = $this->getRowsForId("dc_publisher", $anId);
	foreach ($theRows as $value) {
	    if ($value["place"]) {
		$aRecord->addEntryAsUtf8("publisher", $value["pub_name"] . ", " . $value["place"]);
	    } else {
		$aRecord->addEntryAsUtf8("publisher", $value["pub_name"]);
	    }
	}
    }



    function insertFormat(&$aRecord, $anId) {

	$theRows = $this->getRowsForId("dc_format", $anId);
	foreach ($theRows as $value) {
	    if ($value["SpecReq"]) {
		$aRecord->addEntryAsUtf8("format", $value["format_name"] . " (" . $value["SpecReq"] . ")");
	    } else {
		$aRecord->addEntryAsUtf8("format", $value["format_name"]);
	    }
	}
    }



    function insertRelation(&$aRecord, $anId) {	

	$theRows = $this->getRowsForId("dc_relation", $anId);
	foreach ($theRows as $value) {
	    $mekRelation = "";
	    if ($value["MEK"]) {
		$mekRelation = $value["MEK"];
	    }
	    if ($value["MEKurl"]) {
		if ($mekRelation) {
		    $mekRelation .= " ";
		}
		$mekRelation .= "<" . $value["MEKurl"] . ">";
	    }
	    if ($mekRelation) {
		$aRecord->addEntryAsUtf8("relation", $mekRelation);
	    }
	    $otherRelation = "";
	    if ($value["other"]) {
		$otherRelation = $value["other"];
	    }
	    if ($value["otherurl"]) {
		if ($otherRelation) {
		    $otherRelation .= " ";
		}
		$otherRelation .= "<" . $value["otherurl"] . ">";
	    }
	    if ($otherRelation) {
		$aRecord->addEntryAsUtf8("relation", $otherRelation);
	    }
	}
    }



    function insertSubject(&$aRecord, $anId) {	

	$theRows = $this->getRowsForId("dc_subject", $anId);
	foreach ($theRows as $value) {
	    if ($value["broadtopic"] && $value["topic"] && $value["subtopic"]) {

		// Get the subject names
		$theNames = $this->getRows("broadtopic", "gyujto", $value["broadtopic"]);
		if (count($theNames) != 1) {
		    die("No or multiple broadtopic names");
		}
		$topic = "Topic: " . $theNames[0]["lista"];

		$theNames = $this->getRows("topic_l2", "topic", $value["topic"]);
		if (count($theNames) != 1) {
		    die("No or multiple topic names");
		}
		$topic .= "; " . $theNames[0]["lista"];

		$theNames = $this->getRows("subtopic", "subtopicid", $value["subtopic"]);
		if (count($theNames) != 1) {
		    die("No or multiple subtopic names");
		}
		$topic .= "; " . $theNames[0]["lista"];

		$aRecord->addEntryAsUtf8("subject", $topic);
		$aRecord->addSet($value["broadtopic"] . ":" . $value["topic"] . ":" . $value["subtopic"]);
	    }

	    if ($value["keyword"]) {
		$aRecord->addEntryAsUtf8("subject", $value["keyword"]);
	    }

	    if ($value["period"]) {
		$aRecord->addEntryAsUtf8("coverage", "period: " . $value["period"]);
	    }

	    if ($value["geographic"]) {
		$aRecord->addEntryAsUtf8("coverage", "geographic: " . $value["geographic"]);
	    }
	}
    }


    function insertSimpleField(&$aRecord, $anId, $theField, $theTable, $theColumn, $anIndicator="") {
	$theRows = $this->getRowsForId($theTable, $anId);
	foreach ($theRows as $value) {
	    if ($value[$theColumn]) {
		$aRecord->addEntryAsUtf8($theField, $anIndicator . $value[$theColumn]);
	    }
	}
    }


    function getRecordIdentifier($anId) {
	global $oaiprefix;

	$theIdentifier = $this->getRows("dc_identifier", "id", $anId);
	if (count($theIdentifier) != 1) {
	    die("<h1>No or multiple identifiers for id $anId</h1>");
	    //return $oaiprefix . "MEK-$anId";
	}
	if ($theIdentifier[0]["mekid"]) {
	    return $oaiprefix . $theIdentifier[0]["mekid"];
	}
	die("<h1>No identifier for $anId</h1>");	
	//return $oaiprefix . "MEK-$anId";
    }



    // We have no single update date in the database, therefore we
    // use the youngest date stamp in dc_date. If a record does
    // not have an associated date, we use the earliest date stamp
    // as record date.
    function getDateStamp($anId) {
	global $earliestDatestamp;

	$theRows = $this->doQuery("select max(date) from dc_date where id = \"$anId\"");
	if (count($theRows) != 1) {
	    return $earliestDatestamp;
	}
	return $theRows[0]["max(date)"];
    }



    function &getNextElement() {

	if ($this->numberOfRows == 0) {
	    return NULL;
	}

	/*
	 * Get the id of the next record
	 */
	if ($this->skipOnNext == true) {
	    $aRow = $this->theResult->fetchRow(DB_FETCHMODE_ASSOC, $this->skipRows);
	    $this->skipOnNext = 0;
	} else {
	    $aRow = $this->theResult->fetchRow();
	}
	if (DB::isError($aRow)) {
	    die($aRow->getMessage());
	}
	$this->numberOfRows--;
	// We record the number of records that should be skipped in
	// the result, if we restart from a fresh query
	$this->skipRows++;


	/*
	 * Retrieve the fields of the record with id $recordId
	 */
	$recordId = $aRow["id"];
	$recordIdentifier = $this->getRecordIdentifier($recordId);
	$dateString = $this->getDateStamp($recordId);


	$dcRecord = new DcMetadataRecord($recordIdentifier, $dateString, 0, NULL);

	$this->insertTitle(&$dcRecord, $recordId);
	$this->insertCreator(&$dcRecord, $recordId);
	$this->insertSimpleField(&$dcRecord, $recordId, "creator", "CorporateAuthor", "Cauth_name");
	$this->insertSubject(&$dcRecord, $recordId);
	$this->insertSimpleField(&$dcRecord, $recordId, "description", "note", "state", "State: ");
	$this->insertSimpleField(&$dcRecord, $recordId, "description", "note", "other");
	$this->insertPublisher(&$dcRecord, $recordId);
	$this->insertContributor(&$dcRecord, $recordId);
	$this->insertDate(&$dcRecord, $recordId);
	$this->insertSimpleField(&$dcRecord, $recordId, "type", "dc_type", "dc_type");
	$this->insertFormat(&$dcRecord, $recordId);
	$this->insertSimpleField(&$dcRecord, $recordId, "identifier", "dc_identifier", "mekid");
	$this->insertSimpleField(&$dcRecord, $recordId, "identifier", "dc_identifier", "URL", "URL: ");
	$this->insertSimpleField(&$dcRecord, $recordId, "identifier", "dc_identifier", "ISBN", "ISBN: ");
	$this->insertSimpleField(&$dcRecord, $recordId, "identifier", "dc_identifier", "URN", "URN: ");
	$this->insertSimpleField(&$dcRecord, $recordId, "identifier", "dc_identifier", "ISSN", "ISSN: ");
	$this->insertSimpleField(&$dcRecord, $recordId, "source", "dc_source", "source");
	$this->insertSimpleField(&$dcRecord, $recordId, "source", "PrintedSource", "source", "Printed source: ");
	$this->insertSimpleField(&$dcRecord, $recordId, "language", "dc_language", "m_lang");
	$this->insertRelation(&$dcRecord, $recordId);
	$this->insertSimpleField(&$dcRecord, $recordId, "rights", "dc_rights", "owner", "Owner: ");
	$this->insertSimpleField(&$dcRecord, $recordId, "rights", "dc_rights", "other");

	return $dcRecord;
    }


    function writeTo($fileName) {
	// We delivered $this->skippedRows rows, because we kept track
	// of the delivered rows in $this->skippedRows.
	$fp = fopen ($fileName, 'w');
	fputs($fp, $this->dataSource . "\n"); 
	fputs($fp, $this->theQuery . "\n"); 
	fputs($fp, (string)$this->skipRows . "\n");
	fclose($fp);
    }


    // static method
    function readFrom($fileName) {
	$fp = fopen("$fileName", 'r');
	$dataSource   = trim(fgets($fp));
	$theQuery     = trim(fgets($fp));
	$skipRows     = (int)trim(fgets($fp));
	fclose($fp);
	return new Mek2DcRecordIterator($dataSource, $theQuery, NULL, $skipRows);
    }
}



class Mek2DcMetadataReader extends MetadataRecordReader {


    /*
     * The reader will read its information from the database $dataSource
     */
    function Mek2DcMetadataReader($dataSource) {
	$this->dataSource = $dataSource;
    }


    /*
     * This method queries the database to get all Dublin Core records
     * that comply to the given restrictions. The providerUrl allows to restrict
     * the retrieval to all records from a given URL.
     */
    function getRecords($providerUrl, $from = "", $until = "", $set = "") {

	$theWhereClause = "";

	if ($set) {
	    $theQuery = "select distinct dc_subject.id from dc_subject left join dc_date on dc_subject.id = dc_date.id";
	    $setSpec = split(":", $set, 3);
	    if (count($setSpec) >= 1 && $setSpec[0]) {
		if ($theWhereClause != "") {
		    $theWhereClause .= " and ";
		}
		$theWhereClause .= "dc_subject.broadtopic = \"$setSpec[0]\"";
	    }
	    if (count($setSpec) >= 2 && $setSpec[1]) {
		if ($theWhereClause != "") {
		    $theWhereClause .= " and ";
		}
		$theWhereClause .= "dc_subject.topic = \"$setSpec[1]\"";
	    }
	    if (count($setSpec) >= 3 && $setSpec[2]) {
		if ($theWhereClause != "") {
		    $theWhereClause .= " and ";
		}
		$theWhereClause .= "dc_subject.subtopic = \"$setSpec[2]\"";
	    }
	} else {
	    // :^D
	    $theQuery = "select distinct azonosito.id from azonosito left join dc_date on azonosito.id = dc_date.id";
	}

	
	// TODO: We assume that all records have $earliestDatestamp as
	// date because we have no reliable date information for all
	// record!!
	if ($from) {
	    if ($theWhereClause != "") {
		$theWhereClause .= " and ";
	    }
	    $fr = UTCdatestamp2date($from);
	    $theWhereClause .= "dc_date.date != \"0000-00-00\" and dc_date.date >= \"$fr\"";
	}

	if ($until) {
	    if ($theWhereClause != "") {
		$theWhereClause .= " and ";
	    }
	    $ut = UTCdatestamp2date($until);
	    $theWhereClause .= "dc_date.date != \"0000-00-00\" and dc_date.date <= \"$ut\"";
	}

	if ($theWhereClause) {
	    $theQuery = $theQuery . " where " . $theWhereClause;
	}

	return new Mek2DcRecordIterator($this->dataSource, $theQuery, NULL, 0);
    }



    /*
     * This method queries the database to the record with the unique
     * identifier $identifier.  The providerUrl allows to restrict the
     * retrieval to all records from a given URL.
     */
    function getRecord($providerUrl, $identifier) {
	global $delimiter;

	// Strip oaiprefix from the identifier
	$parts = explode($delimiter, $identifier);
	$identifier = array_pop($parts);
	$theQuery = "select distinct id from dc_identifier where mekid = \"$identifier\"";
	$anIterator = new Mek2DcRecordIterator($this->dataSource, $theQuery, NULL, 0);
	return $anIterator->getNextElement();
    }
}

?>
