<?    
/*
* +----------------------------------------------------------------------+
* | PHP Version 4                                                        |
* +----------------------------------------------------------------------+
* | Copyright (c) 2002 Heinrich Stamerjohanns                            |
* |                                                                      |
* | oaidp-util.php -- Utilities for the OAI Data Provider                |
* |                                                                      |
* | This is free software; you can redistribute it and/or modify it under|
* | the terms of the GNU General Public License as published by the      |
* | Free Software Foundation; either version 2 of the License, or (at    |
* | your option) any later version.                                      |
* | This software is distributed in the hope that it will be useful, but |
* | WITHOUT  ANY WARRANTY; without even the implied warranty of          |
* | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         |
* | GNU General Public License for more details.                         |
* | You should have received a copy of the GNU General Public License    |
* | along with  software; if not, write to the Free Software Foundation, |
* | Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA         |
* |                                                                      |
* +----------------------------------------------------------------------+
* | Derived from work by U. Mller, HUB Berlin                           |
* |                                                                      |
* | Written by Heinrich Stamerjohanns, May 2002                          |
* |            stamer@uni-oldenburg.de                                   |
* +----------------------------------------------------------------------+
*/
//
// $Id: oaidp-util.php,v 1.03 2003/04/08 14:40:23 stamer Exp $
//



include_once "oai2/latin2_utf8.php";



function ReadPrefixFrom($fileName) {
    $fp = fopen("$fileName", 'r');
    $prefix = trim(fgets($fp));
    fclose($fp);
    return $prefix;
}



function WritePrefixTo($fileName, $prefix)
{
    $fp = fopen ($fileName, 'w');
    fputs($fp, $prefix . "\n"); 
    fclose($fp);
}



function get_token() {
    list($usec, $sec) = explode(" ", microtime());
    return ((int)($usec*1000) + (int)($sec*1000));
}



function oai_error($code, $argument='', $value='')
{
    global $request;
    global $request_err;

    switch ($code) {
	case 'badArgument' :
	    $text = "The argument '$argument' (value='$value') included in the request is not valid.";
	    break;
	    
	case 'badGranularity' :
	    $text = "The value '$value' of the argument '$argument' is not valid.";
	    $code = 'badArgument';
	    break;
	    
	case 'badResumptionToken' :
	    $text = "The resumptionToken '$value' does not exist or has already expired.";
	    break;
	    
	case 'badRequestMethod' :
	    $text = "The request method '$argument' is unknown.";
	    $code = 'badVerb';
	    break;
	    
	case 'badVerb' :
	    $text = "The verb '$argument' provided in the request is illegal.";
	    break;
	    
	case 'cannotDisseminateFormat' :
	    $text = "The metadata format '$value' given by $argument is not supported by this repository.";
	    break;
	    
	case 'exclusiveArgument' :
	    $text = 'The usage of resumptionToken as an argument allows no other arguments.';
	    $code = 'badArgument';
	    
	case 'idDoesNotExist' :
	    $text = "The value '$value' of the identifier is illegal for this repository.";
	    break;
	    
	case 'missingArgument' :
	    $text = "The required argument '$argument' is missing in the request.";
	    $code = 'badArgument';
	    break;
	    
	case 'noRecordsMatch' :
	    $text = 'The combination of the given values results in an empty list.';
	    break;
	    
	case 'noMetadataFormats' :
	    $text = 'There are no metadata formats available for the specified item.';
	    break;
	    
	case 'noVerb' :
	    $text = 'The request does not provide any verb.';
	    $code = 'badVerb';
	    break;
	    
	case 'noSetHierarchy' :
	    $text = 'This repository does not support sets.';
	    break;
	    
	case 'sameArgument' :
	    $text = 'Do not use them same argument more than once.';
	    $code = 'badArgument';
	    break;

	case 'sameVerb' :
	    $text = 'Do not use verb more than once.';
	    $code = 'badVerb';
	    break;

	default:
	    $text = "Unknown error: code: '$code', argument: '$argument', value: '$value'";
	    $code = 'badArgument';
    }

    if ($code == 'badVerb' || $code == 'badArgument') {
	$request = $request_err;
    }
    $error .= ' <error code="'.xmlstr($code, false).'">'.xmlstr($text, false)."</error>\n";
    return $error;
}



function xmlstr($string, $xmlescaped="false")
{
    // escape only if string is not escaped
    if (!$xmlescaped) {
	return htmlspecialchars($string, ENT_QUOTES);
    } else {
	return $string;
    }
}



function xmlstrAsUtf8($string, $xmlescaped="false")
{
    // escape only if string is not escaped
    if (!$xmlescaped) {
	return htmlspecialchars(stringToUtf8($string), ENT_QUOTES);
    } else {
	return stringToUtf8($string);
    }
}


function stringToUtf8($string) {
    global $charset;

    if ($charset == "iso-8859-1") {
	return utf8_encode($string);
    } else if ($charset == "iso-8859-2") {
	return latin2_to_utf8($string);
    } else if ($charset == "utf-8") {
	return $string;
    } else {
	die("Unknown character set: '$charset'");
    }
}



// takes either an array or a string and outputs them as XML entities
// this function assumes that the strings are written in utf-8 and have
// not been escaped yet.
function xmlformat($record, $element, $indent = 0)
{
    $str = '';
    if (is_array($record)) {
	foreach  ($record as $val) {
	    $str .= str_pad('', $indent).'<'.$element.'>'.xmlstr(stringToUtf8($val), false).'</'.$element.">\n";
	}
	return $str;
    } elseif ($record != '') {
	return str_pad('', $indent).'<'.$element.'>'.xmlstr(stringToUtf8($record), false).'</'.$element.">\n";
    } else {
	return '';
    }
}



function UTCdatestamp2date($UTCdate) {

    if (strlen($UTCdate) > 10) {
	$date = str_replace('T', ' ', $UTCdate);
	$date = str_replace('Z', '', $date);
	return $date;
    } else {
	return $UTCdate . " 00:00:00";
    }
}



function date2UTCdatestamp($date) {
    global $granularity;

    switch ($granularity) {
	case 'YYYY-MM-DDThh:mm:ssZ':
	    // we assume common date ("YYYY-MM-DD") or 
	    // datetime format ("YYYY-MM-DD hh:mm:ss")
	    // in the database
	    if (strstr($date, ' ')) {
		// date is datetime format
		return str_replace(' ', 'T', $date).'Z';
	    } else {
	        // date is date format
		// granularity 'YYYY-MM-DD' should be used...
		return $date.'T00:00:00Z';
	    }
	    break;

	case 'YYYY-MM-DD':
	    if (strstr($date, ' ')) {
		// date is datetime format
		list($date, $time) = explode(" ", $date);
		return $date;
	    } else {
		return $date;
	    }
	    break;

	default: die("Unknown granularity!");
    }
}


function checkDateFormat($date) {
    global $granularity;
    global $message;

    if ($granularity == 'YYYY-MM-DDThh:mm:ssZ') {
	$checkstr = '([0-9]{4})-([0-9]{1,2})-([0-9]{1,2})T([0-9]{2}):([0-9]{2}):([0-9]{2})Z$';
    } else {
	$checkstr = '([0-9]{4})-([0-9]{1,2})-([0-9]{1,2}$)';
    }
    if (ereg($checkstr, $date, $regs)) {
	if (checkdate($regs[2], $regs[3], $regs[1])) {	
	    return 1;
	}
	else {
	    $message = "Invalid Date: $date is not a valid date.";
	    return 0;
	}
    }
    else {
	$message = "Invalid Date Format: $date does not comply to the date format $granularity.";
	return 0;
    }
}


function formatDatestamp($datestamp)
{
    global $granularity;

    $datestamp = date2UTCdatestamp($datestamp); 
    if (!checkDateFormat($datestamp)) {
	if ($granularity == 'YYYY-MM-DD') {
	    return '2002-01-01';
	} else {
	    return '2002-01-01T00:00:00Z';
	}
    } else {
	return $datestamp;
    }
}

function oai_close() {
    global $compress;

    echo "</OAI-PMH>\n";

    if ($compress) {
	ob_end_flush();
    }
}


function oai_exit() {
    global $CONTENT_TYPE;
    global $xmlheader;
    global $request;
    global $errors;

    header($CONTENT_TYPE);
    echo $xmlheader;
    echo $request;
    echo $errors;
    
    oai_close();
    exit();
}

function php_is_at_least($version) {

    list($c_r, $c_mj, $c_mn) = explode('.', phpversion());
    list($v_r, $v_mj, $v_mn) = explode('.', $version);

    if ($c_r >= $v_r && $c_mj >= $v_mj && $c_mn >= $v_mn)
	return TRUE;
    else
	return FALSE;
}

?>
